@@ -3,19 +3,19 @@
  *****************************************************************************
  * Copyright © 2010-2012 VLC authors and VideoLAN
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU Lesser General Public License as published by
+ * the Free Software Foundation; either version 2.1 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
+ * You should have received a copy of the GNU Lesser General Public License
+ * along with this program; if not, write to the Free Software Foundation,
+ * Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
  *****************************************************************************/
 
 #include <dirent.h>
@@ -422,17 +422,24 @@ static void debug_log(void *data, int level, const libvlc_log_t *ctx, const char
     if (level >= LIBVLC_DEBUG && level <= LIBVLC_ERROR)
         prio = priority[level];
 
+    /* Quit if we are not doing anything */
+    if(!buffer_logging && (!(*verbose) && prio < ANDROID_LOG_ERROR))
+        return;
+
+    /* Add emitting module & type */
+    char* fmt2 = NULL;
+    if(asprintf(&fmt2, "%s %s: %s", ctx->psz_module, ctx->psz_object_type, fmt) < 0)
+        return;
+
     if (buffer_logging) {
         va_list aq;
         va_copy(aq, ap);
-        debug_buffer_log(data, level, fmt, aq);
+        debug_buffer_log(data, level, fmt2, aq);
         va_end(aq);
     }
 
-    if (!*verbose && prio < ANDROID_LOG_ERROR)
-        return;
-
-    __android_log_vprint(prio, "VLC", fmt, ap);
+    __android_log_vprint(prio, "VLC", fmt2, ap);
+    free(fmt2);
 }
 
 void Java_org_videolan_libvlc_LibVLC_startDebugBuffer(JNIEnv *env, jobject thiz)
@@ -481,6 +488,12 @@ void Java_org_videolan_libvlc_LibVLC_nativeInit(JNIEnv *env, jobject thiz)
     methodId = (*env)->GetMethodID(env, cls, "timeStretchingEnabled", "()Z");
     bool enable_time_stretch = (*env)->CallBooleanMethod(env, thiz, methodId);
 
+    methodId = (*env)->GetMethodID(env, cls, "getDeblocking", "()I");
+    int deblocking = (*env)->CallIntMethod(env, thiz, methodId);
+    char deblockstr[2] = "3";
+    snprintf(deblockstr, 2, "%d", deblocking);
+    LOGD("Using deblocking level %d", deblocking);
+
     methodId = (*env)->GetMethodID(env, cls, "getChroma", "()Ljava/lang/String;");
     jstring chroma = (*env)->CallObjectMethod(env, thiz, methodId);
     const char *chromastr = (*env)->GetStringUTFChars(env, chroma, 0);
@@ -506,6 +519,7 @@ void Java_org_videolan_libvlc_LibVLC_nativeInit(JNIEnv *env, jobject thiz)
         "--avcodec-threads=0",
         "--subsdec-encoding", subsencodingstr,
         enable_time_stretch ? "--audio-time-stretch" : "--no-audio-time-stretch",
+        "--avcodec-skiploopfilter", deblockstr,
         use_opensles ? "--aout=opensles" : "--aout=android_audiotrack",
         "--androidsurface-chroma", chromastr != NULL && chromastr[0] != 0 ? chromastr : "RV32",
     };
@@ -1245,6 +1259,20 @@ jint Java_org_videolan_libvlc_LibVLC_setSpuTrack(JNIEnv *env, jobject thiz, jint
     return -1;
 }
 
+jint Java_org_videolan_libvlc_LibVLC_addSubtitleTrack(JNIEnv *env, jobject thiz, jstring path)
+{
+    libvlc_media_player_t *mp = getMediaPlayer(env, thiz);
+    if (mp) {
+        jboolean isCopy;
+        const char* psz_path = (*env)->GetStringUTFChars(env, path, &isCopy);
+        jint res = libvlc_video_set_subtitle_file(mp, psz_path);
+        (*env)->ReleaseStringUTFChars(env, path, psz_path);
+        return res;
+    } else {
+        return -1;
+    }
+}
+
 void Java_org_videolan_libvlc_LibVLC_nativeReadDirectory(JNIEnv *env, jobject thiz, jstring path, jobject arrayList)
 {
     jboolean isCopy;
