@@ -1,21 +1,21 @@
 /*****************************************************************************
  * LibVlcUtil.java
  *****************************************************************************
- * Copyright © 2011-2012 VLC authors and VideoLAN
+ * Copyright © 2011-2013 VLC authors and VideoLAN
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU Lesser General Public License as published by
+ * the Free Software Foundation; either version 2.1 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
+ * You should have received a copy of the GNU Lesser General Public License
+ * along with this program; if not, write to the Free Software Foundation,
+ * Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
  *****************************************************************************/
 
 package org.videolan.libvlc;
@@ -30,10 +30,11 @@ import java.nio.ByteBuffer;
 import java.nio.ByteOrder;
 
 import android.content.Context;
+import android.os.Build;
 import android.util.Log;
 
 public class LibVlcUtil {
-    public final static String TAG = "LibVlc/Util";
+    public final static String TAG = "VLC/LibVLC/Util";
 
     public static boolean isFroyoOrLater()
     {
@@ -80,7 +81,7 @@ public class LibVlcUtil {
 
         String CPU_ABI = android.os.Build.CPU_ABI;
         String CPU_ABI2 = "none";
-        if(android.os.Build.VERSION.SDK_INT >= 8) { // CPU_ABI2 since 2.2
+        if(android.os.Build.VERSION.SDK_INT >= Build.VERSION_CODES.FROYO) { // CPU_ABI2 since 2.2
             try {
                 CPU_ABI2 = (String)android.os.Build.class.getDeclaredField("CPU_ABI2").get(null);
             } catch (Exception e) { }
@@ -91,6 +92,8 @@ public class LibVlcUtil {
         Log.i(TAG, "fpu = " + elf.att_fpu);
         boolean hasNeon = false, hasFpu = false, hasArmV6 = false,
                 hasArmV7 = false, hasMips = false, hasX86 = false;
+        float bogoMIPS = -1;
+        int processors = 0;
 
         if(CPU_ABI.equals("x86")) {
             hasX86 = true;
@@ -126,6 +129,16 @@ public class LibVlcUtil {
                     hasNeon = true;
                 if(!hasFpu && line.contains("vfp"))
                     hasFpu = true;
+                if(line.startsWith("processor"))
+                    processors++;
+                if(bogoMIPS < 0 && line.toLowerCase().contains("bogomips")) {
+                    String[] bogo_parts = line.split(":");
+                    try {
+                        bogoMIPS = Float.parseFloat(bogo_parts[1].trim());
+                    } catch(NumberFormatException e) {
+                        bogoMIPS = -1; // invalid bogomips
+                    }
+                }
             }
             fileReader.close();
         } catch(IOException ex){
@@ -134,6 +147,8 @@ public class LibVlcUtil {
             isCompatible = false;
             return false;
         }
+        if(processors == 0)
+            processors = 1; // possibly borked cpuinfo?
 
         // Enforce proper architecture to prevent problems
         if(elf.e_machine == EM_386 && !hasX86) {
@@ -172,11 +187,36 @@ public class LibVlcUtil {
                 return false;
             }
         }
+
         errorMsg = null;
         isCompatible = true;
+        // Store into MachineSpecs
+        machineSpecs = new MachineSpecs();
+        machineSpecs.hasArmV6 = hasArmV6;
+        machineSpecs.hasArmV7 = hasArmV7;
+        machineSpecs.hasFpu = hasFpu;
+        machineSpecs.hasMips = hasMips;
+        machineSpecs.hasNeon = hasNeon;
+        machineSpecs.hasX86 = hasX86;
+        machineSpecs.bogoMIPS = bogoMIPS;
+        machineSpecs.processors = processors;
         return true;
     }
 
+    public static MachineSpecs getMachineSpecs() {
+        return machineSpecs;
+    }
+    private static MachineSpecs machineSpecs = null;
+    public static class MachineSpecs {
+        public boolean hasNeon;
+        public boolean hasFpu;
+        public boolean hasArmV6;
+        public boolean hasArmV7;
+        public boolean hasMips;
+        public boolean hasX86;
+        public float bogoMIPS;
+        public int processors;
+    }
 
     private static final int EM_386 = 3;
     private static final int EM_MIPS = 8;
