@@ -28,6 +28,8 @@ import java.io.FileOutputStream;
 import java.io.IOException;
 import java.io.InputStreamReader;
 import java.io.OutputStreamWriter;
+import java.net.URI;
+import java.net.URISyntaxException;
 import java.util.ArrayList;
 import java.util.Calendar;
 import java.util.HashMap;
@@ -90,6 +92,7 @@ public class AudioService extends Service {
     public static final String ACTION_REMOTE_STOP = "org.videolan.vlc.remote.Stop";
     public static final String ACTION_REMOTE_FORWARD = "org.videolan.vlc.remote.Forward";
     public static final String ACTION_REMOTE_LAST_PLAYLIST = "org.videolan.vlc.remote.LastPlaylist";
+    public static final String ACTION_WIDGET_INIT = "org.videolan.vlc.widget.INIT";
     public static final String ACTION_WIDGET_UPDATE = "org.videolan.vlc.widget.UPDATE";
     public static final String ACTION_WIDGET_UPDATE_POSITION = "org.videolan.vlc.widget.UPDATE_POSITION";
 
@@ -160,6 +163,7 @@ public class AudioService extends Service {
         filter.addAction(ACTION_REMOTE_STOP);
         filter.addAction(ACTION_REMOTE_FORWARD);
         filter.addAction(ACTION_REMOTE_LAST_PLAYLIST);
+        filter.addAction(ACTION_WIDGET_INIT);
         filter.addAction(Intent.ACTION_HEADSET_PLUG);
         filter.addAction(AudioManager.ACTION_AUDIO_BECOMING_NOISY);
         filter.addAction(VLCApplication.SLEEP_INTENT);
@@ -186,7 +190,7 @@ public class AudioService extends Service {
      * Set up the remote control and tell the system we want to be the default receiver for the MEDIA buttons
      * @see http://android-developers.blogspot.fr/2010/06/allowing-applications-to-play-nicer.html
      */
-    @TargetApi(14)
+    @TargetApi(Build.VERSION_CODES.ICE_CREAM_SANDWICH)
     public void setUpRemoteControlClient() {
         Context context = VLCApplication.getAppContext();
         AudioManager audioManager = (AudioManager)context.getSystemService(AUDIO_SERVICE);
@@ -221,13 +225,22 @@ public class AudioService extends Service {
      *
      * @param p Playback state
      */
-    @TargetApi(14)
-    private void setRemoteControlClientPlaybackState(int p) {
-        if(!Util.isICSOrLater())
+    @TargetApi(Build.VERSION_CODES.ICE_CREAM_SANDWICH)
+    private void setRemoteControlClientPlaybackState(int state) {
+        if(!Util.isICSOrLater() || mRemoteControlClient == null)
             return;
 
-        if (mRemoteControlClient != null)
-            mRemoteControlClient.setPlaybackState(p);
+        switch (state) {
+            case EventHandler.MediaPlayerPlaying:
+                mRemoteControlClient.setPlaybackState(RemoteControlClient.PLAYSTATE_PLAYING);
+                break;
+            case EventHandler.MediaPlayerPaused:
+                mRemoteControlClient.setPlaybackState(RemoteControlClient.PLAYSTATE_PAUSED);
+                break;
+            case EventHandler.MediaPlayerStopped:
+                mRemoteControlClient.setPlaybackState(RemoteControlClient.PLAYSTATE_STOPPED);
+                break;
+        }
     }
 
     @Override
@@ -254,7 +267,7 @@ public class AudioService extends Service {
         return mInterface;
     }
 
-    @TargetApi(8)
+    @TargetApi(Build.VERSION_CODES.FROYO)
     private void changeAudioFocus(boolean gain) {
         if(!Util.isFroyoOrLater()) // NOP if not supported
             return;
@@ -330,6 +343,8 @@ public class AudioService extends Service {
                 next();
             } else if (action.equalsIgnoreCase(ACTION_REMOTE_LAST_PLAYLIST)) {
                 loadLastPlaylist();
+            } else if (action.equalsIgnoreCase(ACTION_WIDGET_INIT)) {
+                updateWidget(context);
             }
 
             /*
@@ -398,7 +413,7 @@ public class AudioService extends Service {
                     }
 
                     service.changeAudioFocus(true);
-                    service.setRemoteControlClientPlaybackState(RemoteControlClient.PLAYSTATE_PLAYING);
+                    service.setRemoteControlClientPlaybackState(EventHandler.MediaPlayerPlaying);
                     if (!service.mWakeLock.isHeld())
                         service.mWakeLock.acquire();
                     break;
@@ -406,14 +421,14 @@ public class AudioService extends Service {
                     Log.i(TAG, "MediaPlayerPaused");
                     service.executeUpdate();
                     service.showNotification();
-                    service.setRemoteControlClientPlaybackState(RemoteControlClient.PLAYSTATE_PAUSED);
+                    service.setRemoteControlClientPlaybackState(EventHandler.MediaPlayerPaused);
                     if (service.mWakeLock.isHeld())
                         service.mWakeLock.release();
                     break;
                 case EventHandler.MediaPlayerStopped:
                     Log.i(TAG, "MediaPlayerStopped");
                     service.executeUpdate();
-                    service.setRemoteControlClientPlaybackState(RemoteControlClient.PLAYSTATE_STOPPED);
+                    service.setRemoteControlClientPlaybackState(EventHandler.MediaPlayerStopped);
                     if (service.mWakeLock.isHeld())
                         service.mWakeLock.release();
                     break;
@@ -434,8 +449,10 @@ public class AudioService extends Service {
                     service.updateWidgetPosition(service, pos);
                     break;
                 case EventHandler.MediaPlayerEncounteredError:
-                    service.showToast(VLCApplication.getAppContext().getString(R.string.invalid_location,
-                            service.mCurrentMedia.getLocation()), Toast.LENGTH_SHORT);
+                    if (service.mCurrentMedia != null) {
+                        service.showToast(service.getString(R.string.invalid_location,
+                                service.mCurrentMedia.getLocation()), Toast.LENGTH_SHORT);
+                    }
                     service.executeUpdate();
                     service.next();
                     if (service.mWakeLock.isHeld())
@@ -575,6 +592,7 @@ public class AudioService extends Service {
                 notification = builder.build();
             }
 
+            startService(new Intent(this, AudioService.class));
             startForeground(3, notification);
         }
         catch (NoSuchMethodError e){
@@ -586,6 +604,7 @@ public class AudioService extends Service {
 
     private void hideNotification() {
         stopForeground(true);
+        stopSelf();
     }
 
     private void pause() {
@@ -608,7 +627,7 @@ public class AudioService extends Service {
     private void stop() {
         mLibVLC.stop();
         mEventHandler.removeHandler(mVlcEventHandler);
-        setRemoteControlClientPlaybackState(RemoteControlClient.PLAYSTATE_STOPPED);
+        setRemoteControlClientPlaybackState(EventHandler.MediaPlayerStopped);
         mCurrentMedia = null;
         mMediaList.clear();
         mPrevious.clear();
@@ -655,7 +674,7 @@ public class AudioService extends Service {
         saveCurrentMedia();
     }
 
-    @TargetApi(14)
+    @TargetApi(Build.VERSION_CODES.ICE_CREAM_SANDWICH)
     private void updateRemoteControlClientMetadata() {
         if(!Util.isICSOrLater()) // NOP check
             return;
@@ -836,15 +855,16 @@ public class AudioService extends Service {
             } else {
                 MediaDatabase db = MediaDatabase.getInstance(AudioService.this);
                 for (int i = 0; i < mediaPathList.size(); i++) {
-                    String path = mediaPathList.get(i);
-                    Media media = db.getMedia(AudioService.this, path);
+                    String location = mediaPathList.get(i);
+                    Media media = db.getMedia(AudioService.this, location);
                     if(media == null) {
-                        if (!validatePath(path)) {
-                            showToast(getResources().getString(R.string.invalid_location, path), Toast.LENGTH_SHORT);
+                        if(!validateLocation(location)) {
+                            Log.w(TAG, "Invalid location " + location);
+                            showToast(getResources().getString(R.string.invalid_location, location), Toast.LENGTH_SHORT);
                             continue;
                         }
-                        Log.v(TAG, "Creating on-the-fly Media object for " + path);
-                        media = new Media(path, false);
+                        Log.v(TAG, "Creating on-the-fly Media object for " + location);
+                        media = new Media(location, false);
                     }
                     mMediaList.add(media);
                 }
@@ -905,9 +925,9 @@ public class AudioService extends Service {
         }
 
         @Override
-        public void append(List<String> mediaPathList) throws RemoteException {
+        public void append(List<String> mediaLocationList) throws RemoteException {
             if (mMediaList.size() == 0) {
-                load(mediaPathList, 0, false, false);
+                load(mediaLocationList, 0, false, false);
                 return;
             }
 
@@ -915,16 +935,16 @@ public class AudioService extends Service {
                 return;
             }
             MediaDatabase db = MediaDatabase.getInstance(AudioService.this);
-            for (int i = 0; i < mediaPathList.size(); i++) {
-                String path = mediaPathList.get(i);
-                Media media = db.getMedia(AudioService.this, path);
+            for (int i = 0; i < mediaLocationList.size(); i++) {
+                String location = mediaLocationList.get(i);
+                Media media = db.getMedia(AudioService.this, location);
                 if(media == null) {
-                    if (!validatePath(path)) {
-                        showToast(getResources().getString(R.string.invalid_location, path), Toast.LENGTH_SHORT);
+                    if (!validateLocation(location)) {
+                        showToast(getResources().getString(R.string.invalid_location, location), Toast.LENGTH_SHORT);
                         continue;
                     }
-                    Log.v(TAG, "Creating on-the-fly Media object for " + path);
-                    media = new Media(path, false);
+                    Log.v(TAG, "Creating on-the-fly Media object for " + location);
+                    media = new Media(location, false);
                 }
                 mMediaList.add(media);
             }
@@ -1019,7 +1039,7 @@ public class AudioService extends Service {
             i.putExtra("artist", mCurrentMedia.getArtist());
         }
         else {
-            i.putExtra("title", "VLC mini player");
+            i.putExtra("title", context.getString(R.string.widget_name));
             i.putExtra("artist", "");
         }
         i.putExtra("isplaying", mLibVLC.isPlaying());
@@ -1132,14 +1152,19 @@ public class AudioService extends Service {
         }
     }
 
-    private boolean validatePath(String path)
+    private boolean validateLocation(String location)
     {
         /* Check if the MRL contains a scheme */
-        if (!path.matches("\\w+://.+"))
-            path = "file://".concat(path);
-        if (path.toLowerCase(Locale.ENGLISH).startsWith("file://")) {
+        if (!location.matches("\\w+://.+"))
+            location = "file://".concat(location);
+        if (location.toLowerCase(Locale.ENGLISH).startsWith("file://")) {
             /* Ensure the file exists */
-            File f = new File(path);
+            File f;
+            try {
+                f = new File(new URI(location));
+            } catch (URISyntaxException e) {
+                return false;
+            }
             if (!f.isFile())
                 return false;
         }
