@@ -20,6 +20,7 @@
 
 package org.videolan.vlc.gui;
 
+import android.os.Message;
 import org.videolan.libvlc.LibVlcException;
 import org.videolan.libvlc.LibVlcUtil;
 import org.videolan.vlc.AudioService;
@@ -28,11 +29,13 @@ import org.videolan.vlc.MediaLibrary;
 import org.videolan.vlc.R;
 import org.videolan.vlc.Util;
 import org.videolan.vlc.VLCCallbackTask;
+import org.videolan.vlc.WeakHandler;
 import org.videolan.vlc.gui.SidebarAdapter.SidebarEntry;
 import org.videolan.vlc.gui.video.VideoListAdapter;
 import org.videolan.vlc.interfaces.ISortable;
 import org.videolan.vlc.widget.AudioMiniPlayer;
 
+import android.annotation.TargetApi;
 import android.app.AlertDialog;
 import android.app.Dialog;
 import android.content.BroadcastReceiver;
@@ -45,10 +48,12 @@ import android.content.SharedPreferences.Editor;
 import android.content.pm.PackageInfo;
 import android.content.pm.PackageManager.NameNotFoundException;
 import android.content.res.Configuration;
+import android.os.Build;
 import android.os.Bundle;
 import android.os.Handler;
 import android.preference.PreferenceManager;
 import android.support.v4.app.Fragment;
+import android.support.v4.app.FragmentManager;
 import android.support.v4.app.FragmentTransaction;
 import android.util.Log;
 import android.view.Display;
@@ -85,6 +90,7 @@ public class MainActivity extends SherlockFragmentActivity {
     private static final String PREF_FIRST_RUN = "first_run";
 
     private static final int ACTIVITY_RESULT_PREFERENCES = 1;
+    private static final int ACTIVITY_SHOW_INFOLAYOUT = 2;
 
     private ActionBar mActionBar;
     private SlidingMenu mMenu;
@@ -103,6 +109,8 @@ public class MainActivity extends SherlockFragmentActivity {
     private boolean mFirstRun = false;
     private boolean mScanNeeded = true;
 
+    private Handler mHandler = new MainActivityHandler(this);
+
     @Override
     protected void onCreate(Bundle savedInstanceState) {
         if (!LibVlcUtil.hasCompatibleCPU(this)) {
@@ -177,16 +185,7 @@ public class MainActivity extends SherlockFragmentActivity {
         mInfoText = (TextView) v_main.findViewById(R.id.info_text);
 
         /* Set up the action bar */
-        mActionBar = getSupportActionBar();
-        mActionBar.setNavigationMode(ActionBar.NAVIGATION_MODE_STANDARD);
-        mActionBar.setDisplayOptions(0, ActionBar.DISPLAY_SHOW_TITLE);
-        mActionBar.setDisplayHomeAsUpEnabled(true);
-
-        /* Add padding between the home button and the arrow */
-        ImageView home = (ImageView)findViewById(Util.isHoneycombOrLater()
-                ? android.R.id.home : R.id.abs__home);
-        if (home != null)
-            home.setPadding(20, 0, 0, 0);
+        prepareActionBar();
 
         /* Set up the sidebar click listener */
         listView.setOnItemClickListener(new OnItemClickListener() {
@@ -207,23 +206,11 @@ public class MainActivity extends SherlockFragmentActivity {
                  * activating an old backstack, when a user hits the back button
                  * to quit
                  */
-                for(int i = 0; i < getSupportFragmentManager().getBackStackEntryCount(); i++) {
-                    getSupportFragmentManager().popBackStack();
-                }
-
-                /**
-                 * Do not move this getFragment("audio")!
-                 * This is to ensure that if audio is not already loaded, it
-                 * will be loaded ahead of the detach/attach below.
-                 * Otherwise if you add() a fragment after an attach/detach,
-                 * it will take over the placeholder and you will end up with
-                 * the audio fragment when some other fragment should be there.
-                 */
-                Fragment audioFragment = getFragment("audio");
+                getSupportFragmentManager().popBackStack(null, FragmentManager.POP_BACK_STACK_INCLUSIVE);
 
+                /* Switch the fragment */
                 FragmentTransaction ft = getSupportFragmentManager().beginTransaction();
-                ft.detach(current);
-                ft.attach(getFragment(entry.id));
+                ft.replace(R.id.fragment_placeholder, getFragment(entry.id), entry.id);
                 ft.commit();
                 mCurrentFragment = entry.id;
 
@@ -235,7 +222,7 @@ public class MainActivity extends SherlockFragmentActivity {
                 getFragment(mCurrentFragment).setUserVisibleHint(true);
                 // HACK ALERT: Set underlying audio browser to be invisible too.
                 if(current.getTag().equals("tracks"))
-                    audioFragment.setUserVisibleHint(false);
+                    getFragment("audio").setUserVisibleHint(false);
 
                 mMenu.showContent();
             }
@@ -260,8 +247,7 @@ public class MainActivity extends SherlockFragmentActivity {
              * the info dialog. If (for any reason) the dialog is not shown,
              * open the menu after a short delay.
              */
-            final Handler handler = new Handler();
-            handler.postDelayed(new Runnable() {
+            mHandler.postDelayed(new Runnable() {
                 @Override
                 public void run() {
                     mMenu.showMenu();
@@ -288,6 +274,20 @@ public class MainActivity extends SherlockFragmentActivity {
         mMenu.setBehindOffset(Util.convertDpToPx(behindOffset_dp));
     }
 
+    @TargetApi(Build.VERSION_CODES.HONEYCOMB)
+    private void prepareActionBar() {
+        mActionBar = getSupportActionBar();
+        mActionBar.setNavigationMode(ActionBar.NAVIGATION_MODE_STANDARD);
+        mActionBar.setDisplayOptions(0, ActionBar.DISPLAY_SHOW_TITLE);
+        mActionBar.setDisplayHomeAsUpEnabled(true);
+
+        /* Add padding between the home button and the arrow */
+        ImageView home = (ImageView)findViewById(Util.isHoneycombOrLater()
+                ? android.R.id.home : R.id.abs__home);
+        if (home != null)
+            home.setPadding(20, 0, 0, 0);
+    }
+
     @Override
     protected void onResume() {
         super.onResume();
@@ -322,7 +322,6 @@ public class MainActivity extends SherlockFragmentActivity {
             found = true;
         }
 
-        mSidebarAdapter.lockSemaphore();
         /**
          * Let's see if Android recreated anything for us in the bundle.
          * Prevent duplicate creation of fragments, since mSidebarAdapter might
@@ -336,7 +335,6 @@ public class MainActivity extends SherlockFragmentActivity {
                 mSidebarAdapter.restoreFragment(fragmentTag, fragment);
             }
         }
-        mSidebarAdapter.unlockSemaphore();
 
         /**
          * Restore the last view.
@@ -354,9 +352,7 @@ public class MainActivity extends SherlockFragmentActivity {
             Log.d(TAG, "Reloading displayed fragment");
             Fragment ff = getFragment(mCurrentFragment);
             FragmentTransaction ft = getSupportFragmentManager().beginTransaction();
-            if(current != null)
-                ft.detach(current);
-            ft.attach(ff);
+            ft.replace(R.id.fragment_placeholder, ff, mCurrentFragment);
             ft.commit();
         }
     }
@@ -417,21 +413,7 @@ public class MainActivity extends SherlockFragmentActivity {
 
     private Fragment getFragment(String id)
     {
-        Fragment fragment = mSidebarAdapter.fetchFragment(id);
-
-        // Prevent fragment from being added twice. (IllegalStateException)
-        // See http://stackoverflow.com/questions/13745787/fragment-already-added-support-lib
-        mSidebarAdapter.lockSemaphore();
-        if(!mSidebarAdapter.isFragmentAdded(id)) {
-            getSupportFragmentManager()
-                .beginTransaction()
-                .add(R.id.fragment_placeholder, fragment, id)
-                .commitAllowingStateLoss();
-            mSidebarAdapter.setFragmentAdded(id);
-        }
-        mSidebarAdapter.unlockSemaphore();
-
-        return fragment;
+        return mSidebarAdapter.fetchFragment(id);
     }
 
     /** Create menu from XML
@@ -590,7 +572,38 @@ public class MainActivity extends SherlockFragmentActivity {
                 mInfoText.setText(info);
                 mInfoProgress.setMax(max);
                 mInfoProgress.setProgress(progress);
-                mInfoLayout.setVisibility(info != null ? View.VISIBLE : View.GONE);
+
+                if (info == null) {
+                    /* Cancel any upcoming visibility change */
+                    mHandler.removeMessages(ACTIVITY_SHOW_INFOLAYOUT);
+                    mInfoLayout.setVisibility(View.GONE);
+                }
+                else {
+                    /* Slightly delay the appearance of the progress bar to avoid unnecessary flickering */
+                    if (!mHandler.hasMessages(ACTIVITY_SHOW_INFOLAYOUT)) {
+                        Message m = new Message();
+                        m.what = ACTIVITY_SHOW_INFOLAYOUT;
+                        mHandler.sendMessageDelayed(m, 300);
+                    }
+                }
+            }
+        }
+    };
+
+    private static class MainActivityHandler extends WeakHandler<MainActivity> {
+        public MainActivityHandler(MainActivity owner) {
+            super(owner);
+        }
+
+        @Override
+        public void handleMessage(Message msg) {
+            MainActivity ma = getOwner();
+            if(ma == null) return;
+
+            switch (msg.what) {
+                case ACTIVITY_SHOW_INFOLAYOUT:
+                    ma.mInfoLayout.setVisibility(View.VISIBLE);
+                    break;
             }
         }
     };
