@@ -53,6 +53,7 @@ public class PreferencesActivity extends PreferenceActivity implements OnSharedP
     public final static String NAME = "VlcSharedPreferences";
     public final static String LAST_MEDIA = "LastMedia";
     public final static String VIDEO_RESUME_TIME = "VideoResumeTime";
+    public final static String VIDEO_SUBTITLE_FILES = "VideoSubtitleFiles";
     public final static int RESULT_RESCAN = RESULT_FIRST_USER + 1;
 
     @Override
@@ -203,6 +204,7 @@ public class PreferencesActivity extends PreferenceActivity implements OnSharedP
                 || key.equalsIgnoreCase("subtitles_text_encoding")
                 || key.equalsIgnoreCase("aout")
                 || key.equalsIgnoreCase("enable_time_stretching_audio")
+                || key.equalsIgnoreCase("deblocking")
                 || key.equalsIgnoreCase("chroma_format")
                 || key.equalsIgnoreCase("enable_verbose_mode")) {
             Util.updateLibVlcSettings(sharedPreferences);
