@@ -26,6 +26,7 @@ import org.videolan.vlc.R;
 import org.videolan.vlc.RepeatType;
 import org.videolan.vlc.Util;
 import org.videolan.vlc.gui.CommonDialogs;
+import org.videolan.vlc.gui.CommonDialogs.MenuType;
 import org.videolan.vlc.gui.MainActivity;
 import org.videolan.vlc.interfaces.IAudioPlayer;
 
@@ -170,8 +171,11 @@ public class AudioPlayerActivity extends Activity implements IAudioPlayer {
 
     @Override
     public void update() {
-        // Exit the player when there is no media
+        // Exit the player and return to the main menu when there is no media
         if (!mAudioController.hasMedia()) {
+            Intent intent = new Intent(this, MainActivity.class);
+            intent.setFlags(Intent.FLAG_ACTIVITY_REORDER_TO_FRONT);
+            startActivity(intent);
             finish();
             return;
         }
@@ -321,7 +325,7 @@ public class AudioPlayerActivity extends Activity implements IAudioPlayer {
     	return super.onKeyDown(keyCode, event);
     }
 
-    public void showAdvanceFunction(View v) {
-        CommonDialogs.advancedOptions(this, v);
+    public void showAdvancedOptions(View v) {
+        CommonDialogs.advancedOptions(this, v, MenuType.Audio);
     }
 }
