@@ -20,10 +20,18 @@
 
 package org.videolan.vlc.gui.video;
 
+import java.io.ByteArrayInputStream;
+import java.io.ByteArrayOutputStream;
 import java.io.File;
+import java.io.IOException;
+import java.io.ObjectInputStream;
+import java.io.ObjectOutputStream;
+import java.io.StreamCorruptedException;
 import java.io.UnsupportedEncodingException;
 import java.lang.reflect.Method;
 import java.net.URLDecoder;
+import java.util.ArrayList;
+import java.util.Date;
 import java.util.Map;
 
 import org.videolan.libvlc.EventHandler;
@@ -38,6 +46,7 @@ import org.videolan.vlc.Util;
 import org.videolan.vlc.VLCApplication;
 import org.videolan.vlc.WeakHandler;
 import org.videolan.vlc.gui.CommonDialogs;
+import org.videolan.vlc.gui.CommonDialogs.MenuType;
 import org.videolan.vlc.gui.PreferencesActivity;
 import org.videolan.vlc.gui.audio.AudioPlayerActivity;
 import org.videolan.vlc.interfaces.IPlayerControl;
@@ -91,9 +100,12 @@ import android.widget.SeekBar.OnSeekBarChangeListener;
 import android.widget.TextView;
 
 public class VideoPlayerActivity extends Activity implements IVideoPlayer {
-
     public final static String TAG = "VLC/VideoPlayerActivity";
 
+    // Internal intent identifier to distinguish between internal launch and
+    // external intent.
+    private final static String PLAY_FROM_VIDEOGRID = "org.videolan.vlc.gui.video.PLAY_FROM_VIDEOGRID";
+
     private SurfaceView mSurface;
     private SurfaceHolder mSurfaceHolder;
     private FrameLayout mSurfaceFrame;
@@ -175,9 +187,15 @@ public class VideoPlayerActivity extends Activity implements IVideoPlayer {
     // Tracks & Subtitles
     private Map<Integer,String> mAudioTracksList;
     private Map<Integer,String> mSubtitleTracksList;
+    /**
+     * Used to store a selected subtitle; see onActivityResult.
+     * It is possible to have multiple custom subs in one session
+     * (just like desktop VLC allows you as well.)
+     */
+    private ArrayList<String> mSubtitleSelectedFiles = new ArrayList<String>();
 
     @Override
-    @TargetApi(11)
+    @TargetApi(Build.VERSION_CODES.HONEYCOMB)
     protected void onCreate(Bundle savedInstanceState) {
         super.onCreate(savedInstanceState);
         setContentView(R.layout.player);
@@ -289,6 +307,10 @@ public class VideoPlayerActivity extends Activity implements IVideoPlayer {
         SharedPreferences preferences = getSharedPreferences(PreferencesActivity.NAME, MODE_PRIVATE);
         SharedPreferences.Editor editor = preferences.edit();
         editor.putLong(PreferencesActivity.VIDEO_RESUME_TIME, -1);
+        // Also clear the subs list, because it is supposed to be per session
+        // only (like desktop VLC). We don't want the customs subtitle file
+        // to persist forever with this video.
+        editor.putString(PreferencesActivity.VIDEO_SUBTITLE_FILES, null);
         editor.commit();
 
         IntentFilter filter = new IntentFilter();
@@ -360,10 +382,10 @@ public class VideoPlayerActivity extends Activity implements IVideoPlayer {
 
         mSurface.setKeepScreenOn(false);
 
+        SharedPreferences preferences = getSharedPreferences(PreferencesActivity.NAME, MODE_PRIVATE);
+        SharedPreferences.Editor editor = preferences.edit();
         // Save position
         if (time >= 0) {
-            SharedPreferences preferences = getSharedPreferences(PreferencesActivity.NAME, MODE_PRIVATE);
-            SharedPreferences.Editor editor = preferences.edit();
             if(MediaDatabase.getInstance(this).mediaItemExists(mLocation)) {
                 editor.putString(PreferencesActivity.LAST_MEDIA, mLocation);
                 MediaDatabase.getInstance(this).updateMedia(
@@ -374,9 +396,21 @@ public class VideoPlayerActivity extends Activity implements IVideoPlayer {
                 // Video file not in media library, store time just for onResume()
                 editor.putLong(PreferencesActivity.VIDEO_RESUME_TIME, time);
             }
-            editor.commit();
         }
+        // Save selected subtitles
+        String subtitleList_serialized = null;
+        if(mSubtitleSelectedFiles.size() > 0) {
+            Log.d(TAG, "Saving selected subtitle files");
+            ByteArrayOutputStream bos = new ByteArrayOutputStream();
+            try {
+                ObjectOutputStream oos = new ObjectOutputStream(bos);
+                oos.writeObject(mSubtitleSelectedFiles);
+                subtitleList_serialized = bos.toString();
+            } catch(IOException e) {}
+        }
+        editor.putString(PreferencesActivity.VIDEO_SUBTITLE_FILES, subtitleList_serialized);
 
+        editor.commit();
         AudioServiceController.getInstance().unbindAudioService(this);
     }
 
@@ -423,6 +457,46 @@ public class VideoPlayerActivity extends Activity implements IVideoPlayer {
             }}, 500);
 
         showOverlay();
+
+        // Add any selected subtitle file from the file picker
+        if(mSubtitleSelectedFiles.size() > 0) {
+            for(String file : mSubtitleSelectedFiles) {
+                Log.i(TAG, "Adding user-selected subtitle " + file);
+                mLibVLC.addSubtitleTrack(file);
+            }
+            /** FIXME remove when #7540 ES are not updated mid-stream
+             *  a.k.a. subtitle events is fixed
+             */
+            mHandler.postDelayed(new Runnable() {
+                @Override
+                public void run() {
+                    setESTrackLists(true);
+
+                    mHandler.postDelayed(new Runnable() {
+                        @Override
+                        public void run() {
+                            setESTrackLists(true);
+                        }}, 1200);
+                }}, 1000);
+        }
+    }
+
+    @Override
+    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
+        if(data == null) return;
+
+        if(data.getDataString() == null) {
+            Log.d(TAG, "Subtitle selection dialog was cancelled");
+        }
+        if(data.getData() == null) return;
+
+        String uri = data.getData().getPath();
+        if(requestCode == CommonDialogs.INTENT_SPECIFIC) {
+            Log.d(TAG, "Specific subtitle file: " + uri);
+        } else if(requestCode == CommonDialogs.INTENT_GENERIC) {
+            Log.d(TAG, "Generic subtitle file: " + uri);
+        }
+        mSubtitleSelectedFiles.add(data.getData().getPath());
     }
 
     public static void start(Context context, String location) {
@@ -439,6 +513,7 @@ public class VideoPlayerActivity extends Activity implements IVideoPlayer {
 
     public static void start(Context context, String location, String title, Boolean dontParse, Boolean fromStart) {
         Intent intent = new Intent(context, VideoPlayerActivity.class);
+        intent.setAction(VideoPlayerActivity.PLAY_FROM_VIDEOGRID);
         intent.putExtra("itemLocation", location);
         intent.putExtra("itemTitle", title);
         intent.putExtra("dontParse", dontParse);
@@ -937,7 +1012,7 @@ public class VideoPlayerActivity extends Activity implements IVideoPlayer {
 
         // Set Brightness
         getWindow().setAttributes(lp);
-        showInfo(getString(R.string.brightness) + Math.round(lp.screenBrightness*15),1000);
+        showInfo(getString(R.string.brightness) + '\u00A0' + Math.round(lp.screenBrightness*15),1000);
     }
 
     /**
@@ -1264,7 +1339,7 @@ public class VideoPlayerActivity extends Activity implements IVideoPlayer {
      * Dim the status bar and/or navigation icons when needed on Android 3.x.
      * Hide it on Android 4.0 and later
      */
-    @TargetApi(11)
+    @TargetApi(Build.VERSION_CODES.ICE_CREAM_SANDWICH)
     private void dimStatusBar(boolean dim) {
         if (!Util.isHoneycombOrLater() || !Util.hasNavBar())
             return;
@@ -1297,7 +1372,7 @@ public class VideoPlayerActivity extends Activity implements IVideoPlayer {
         mControls.setSeekable(length > 0);
         mSeekbar.setMax(length);
         mSeekbar.setProgress(time);
-        mSysTime.setText(DateFormat.format("kk:mm", System.currentTimeMillis()));
+        mSysTime.setText(DateFormat.getTimeFormat(this).format(new Date(System.currentTimeMillis())));
         mTime.setText(Util.millisToString(time));
         mLength.setText(mDisplayRemainingTime && length > 0
                 ? "- " + Util.millisToString(length - time)
@@ -1318,7 +1393,11 @@ public class VideoPlayerActivity extends Activity implements IVideoPlayer {
     }
 
     private void setESTrackLists() {
-        if(mAudioTracksList == null) {
+        setESTrackLists(false);
+    }
+
+    private void setESTrackLists(boolean force) {
+        if(mAudioTracksList == null || force) {
             if (mLibVLC.getAudioTracksCount() > 2) {
                 mAudioTracksList = mLibVLC.getAudioTrackDescription();
                 mAudioTrack.setOnClickListener(mAudioTrackListener);
@@ -1329,7 +1408,7 @@ public class VideoPlayerActivity extends Activity implements IVideoPlayer {
                 mAudioTrack.setOnClickListener(null);
             }
         }
-        if (mSubtitleTracksList == null) {
+        if (mSubtitleTracksList == null || force) {
             if (mLibVLC.getSpuTracksCount() > 0) {
                 mSubtitleTracksList = mLibVLC.getSpuTrackDescription();
                 mSubtitle.setOnClickListener(mSubtitlesListener);
@@ -1360,15 +1439,17 @@ public class VideoPlayerActivity extends Activity implements IVideoPlayer {
     }
 
     /**
-     *
+     * External extras:
+     * - position (long) - position of the video to start with (in ms)
      */
-    @SuppressWarnings("deprecation")
+    @SuppressWarnings({ "deprecation", "unchecked" })
     private void load() {
         mLocation = null;
         String title = getResources().getString(R.string.title);
         boolean dontParse = false;
         boolean fromStart = false;
         String itemTitle = null;
+        long intentPosition = -1; // position passed in by intent (ms)
 
         if (getIntent().getAction() != null
                 && getIntent().getAction().equals(Intent.ACTION_VIEW)) {
@@ -1390,7 +1471,10 @@ public class VideoPlayerActivity extends Activity implements IVideoPlayer {
                 // Plain URI
                 mLocation = getIntent().getDataString();
             }
-        } else if(getIntent().getExtras() != null) {
+            if(getIntent().getExtras() != null)
+                intentPosition = getIntent().getExtras().getLong("position", -1);
+        } else if(getIntent().getAction() != null
+                && getIntent().getAction().equals(PLAY_FROM_VIDEOGRID) && getIntent().getExtras() != null) {
             /* Started from VideoListActivity */
             mLocation = getIntent().getExtras().getString("itemLocation");
             itemTitle = getIntent().getExtras().getString("itemTitle");
@@ -1409,6 +1493,7 @@ public class VideoPlayerActivity extends Activity implements IVideoPlayer {
 
         if (mLocation != null && mLocation.length() > 0 && !dontParse) {
             // restore last position
+            SharedPreferences preferences = getSharedPreferences(PreferencesActivity.NAME, MODE_PRIVATE);
             Media media = MediaDatabase.getInstance(this).getMedia(this, mLocation);
             if(media != null) {
                 // in media library
@@ -1419,15 +1504,33 @@ public class VideoPlayerActivity extends Activity implements IVideoPlayer {
                 mLastSpuTrack = media.getSpuTrack();
             } else {
                 // not in media library
-                SharedPreferences preferences = getSharedPreferences(PreferencesActivity.NAME, MODE_PRIVATE);
                 long rTime = preferences.getLong(PreferencesActivity.VIDEO_RESUME_TIME, -1);
                 SharedPreferences.Editor editor = preferences.edit();
                 editor.putLong(PreferencesActivity.VIDEO_RESUME_TIME, -1);
                 editor.commit();
                 if(rTime > 0)
                     mLibVLC.setTime(rTime);
+
+                if(intentPosition > 0)
+                    mLibVLC.setTime(intentPosition);
             }
 
+            String subtitleList_serialized = preferences.getString(PreferencesActivity.VIDEO_SUBTITLE_FILES, null);
+            ArrayList<String> prefsList = new ArrayList<String>();
+            if(subtitleList_serialized != null) {
+                ByteArrayInputStream bis = new ByteArrayInputStream(subtitleList_serialized.getBytes());
+                try {
+                    ObjectInputStream ois = new ObjectInputStream(bis);
+                    prefsList = (ArrayList<String>)ois.readObject();
+                } catch(ClassNotFoundException e) {}
+                  catch (StreamCorruptedException e) {}
+                  catch (IOException e) {}
+            }
+            for(String x : prefsList){
+                if(!mSubtitleSelectedFiles.contains(x))
+                    mSubtitleSelectedFiles.add(x);
+             }
+
             try {
                 title = URLDecoder.decode(mLocation, "UTF-8");
             } catch (UnsupportedEncodingException e) {
@@ -1449,7 +1552,7 @@ public class VideoPlayerActivity extends Activity implements IVideoPlayer {
     private int getScreenRotation(){
         WindowManager wm = (WindowManager) getSystemService(Context.WINDOW_SERVICE);
         Display display = wm.getDefaultDisplay();
-        if (Build.VERSION.SDK_INT >= 8 /* Android 2.2 has getRotation */) {
+        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.FROYO /* Android 2.2 has getRotation */) {
             try {
                 Method m = display.getClass().getDeclaredMethod("getRotation");
                 return (Integer) m.invoke(display);
@@ -1461,6 +1564,7 @@ public class VideoPlayerActivity extends Activity implements IVideoPlayer {
         }
     }
 
+    @TargetApi(Build.VERSION_CODES.GINGERBREAD)
     private int getScreenOrientation (){
         switch (getScreenRotation()) {
         case Surface.ROTATION_0:
@@ -1469,12 +1573,12 @@ public class VideoPlayerActivity extends Activity implements IVideoPlayer {
             return ActivityInfo.SCREEN_ORIENTATION_LANDSCAPE;
         case Surface.ROTATION_180:
             // SCREEN_ORIENTATION_REVERSE_PORTRAIT only available since API Level 9+
-             return (Build.VERSION.SDK_INT >= 8
+             return (Build.VERSION.SDK_INT >= Build.VERSION_CODES.FROYO
                     ? ActivityInfo.SCREEN_ORIENTATION_REVERSE_PORTRAIT
                     : ActivityInfo.SCREEN_ORIENTATION_PORTRAIT);
         case Surface.ROTATION_270:
             // SCREEN_ORIENTATION_REVERSE_LANDSCAPE only available since API Level 9+
-            return (Build.VERSION.SDK_INT >= 8
+            return (Build.VERSION.SDK_INT >= Build.VERSION_CODES.FROYO
                     ? ActivityInfo.SCREEN_ORIENTATION_REVERSE_LANDSCAPE
                     : ActivityInfo.SCREEN_ORIENTATION_LANDSCAPE);
         default :
@@ -1482,7 +1586,7 @@ public class VideoPlayerActivity extends Activity implements IVideoPlayer {
         }
     }
 
-    public void showAdvanceFunction(View v) {
-        CommonDialogs.advancedOptions(this, v);
+    public void showAdvancedOptions(View v) {
+        CommonDialogs.advancedOptions(this, v, MenuType.Video);
     }
 }
